package com.zy.bms.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zy.bms.common.Constant;
import com.zy.bms.entity.Logs;
import com.zy.bms.mapper.LogsMapper;
import com.zy.bms.pojo.dto.MqttDTO;
import com.zy.bms.pojo.io.LogsIO;
import com.zy.bms.pojo.vo.LogsVo;
import com.zy.bms.service.ILogsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * MQTT 收发日志 Service
 *
 * @author chenyi
 * Create on 2020/4/10
 */
@Slf4j
@Service
public class LogsService extends ServiceImpl<LogsMapper, Logs> implements ILogsService {

    private static final Set<String> UBI_SET = new HashSet<>(Arrays.asList("21", "23", "26", "27", "28", "29", "30", "80"));
    private static final Set<String> LP_SET = new HashSet<>(Arrays.asList("70", "71", "72"));

    @Override
    public void saveReceive(String topic, String payload) {
        saveCommon(topic, payload, 0);
    }

    @Override
    public void saveSend(String topic, String payload) {
        saveCommon(topic, payload, 1);
    }

    @Override
    public IPage<LogsVo> listPage(LogsIO io) {
        Page<LogsVo> page = new Page<>(io.getCurrent(), io.getSize());
        return baseMapper.listPage(page, io);
    }

    /**
     * 保存日志记录
     *
     * @param topic   主题
     * @param content 内容
     * @param type    类型 0:接收 1:发送
     */
    private void saveCommon(String topic, String content, Integer type) {
        MqttDTO tmp = MqttDTO.parse(content);
        //创建日志对象
        Logs entity = Logs.builder().topic(topic).type(type).m(tmp.getM()).content(content).build();
        if (UBI_SET.contains(tmp.getM())) {
            entity.setDeviceType(Constant.DEVICE_TYPE.UBI);
        } else if (LP_SET.contains(tmp.getM())) {
            entity.setDeviceType(Constant.DEVICE_TYPE.LP);
        } else {
            entity.setDeviceType(tmp.getString("type"));
        }
        String openNum = Constant.splitOpenNum(topic);
        entity.setOpenNum(openNum == null ? tmp.getString("regnum") : openNum);
        baseMapper.insert(entity);
    }
}
